package com.cf.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Collectors;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.ExampleMatcher.GenericPropertyMatcher;
import org.springframework.data.domain.ExampleMatcher.MatcherConfigurer;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import com.cf.entity.CommentEntity;
import com.cf.entity.RestaurantEntity;
import com.cf.entity.UserEntity;
import com.cf.repository.CommentRepository;
import com.cf.repository.RestaurantRepository;
import com.cf.repository.UserRepository;

import jdk.internal.jline.internal.Log;


@RestController
public class ApplicationController {

	@Autowired
	RestaurantRepository restaurantRepository;
	
	@Autowired
	UserRepository userRepository;
	
	@Autowired
	CommentRepository commentRepository;
	
	@GetMapping("/findAllRestaurant")
	public List getAll() {
	return restaurantRepository.findAll();
	}
	
	@RequestMapping(value = "/findRestaurantById", method = RequestMethod.GET)
	public @ResponseBody RestaurantEntity getOne(
			@RequestParam(value = "id")Long id
			) {
	return restaurantRepository.findById(id).get();
	}

	@RequestMapping(value = "/login", method = RequestMethod.GET)
	public @ResponseBody UserEntity login(
			@RequestParam(value = "user")String usuario,
			@RequestParam(value = "password")String contrasena
			) {
		List lista = userRepository.login(usuario, contrasena);
	return (UserEntity) lista.get(0);
	}
	
	@RequestMapping(value = "/findUserById", method = RequestMethod.GET)
	public @ResponseBody UserEntity getOneUser(
			@RequestParam(value = "id")Long id
			) {
	return userRepository.findById(id).get();
	}
	
	@RequestMapping(value = "/addUser", method = RequestMethod.POST, consumes="application/json")
	public @ResponseBody UserEntity addUser(
			@RequestBody UserEntity body) {
		userRepository.addUser(body.getNombre_usuario(),
				body.getCorreo(),
				body.getProvincia(),
				body.getContrasena());
	List lista = userRepository.login(body.getNombre_usuario(), body.getContrasena());
	return (UserEntity) lista.get(0);
	}
	
	@RequestMapping(value = "/updateUser", method = RequestMethod.POST, consumes="application/json")
	public @ResponseBody Optional<UserEntity> updateUser(
			@RequestBody UserEntity body) {
		 userRepository.updateUser(body.getId_usuario(),
				body.getNombre_usuario(),
				body.getCorreo(),
				body.getProvincia());
		 return userRepository.findById(body.getId_usuario());
	}
	
	@RequestMapping(value = "/changePassword", method = RequestMethod.POST, consumes="application/json")
	public @ResponseBody UserEntity changePassword(
			@RequestBody UserEntity body) {
		userRepository.changePassword(body.getId_usuario(),
				body.getContrasena());
		return null;
	}
	
	@RequestMapping(value = "/findCommentsByRestId", method = RequestMethod.GET)
	public @ResponseBody List<CommentEntity> getCommentByRestId(
			@RequestParam(value = "id_restaurante")Long id
			) {
	return commentRepository.findCommentsByRestId(id);
	}
	@RequestMapping(value = "/findRestaurantInCity", method = RequestMethod.GET)
	public @ResponseBody List<RestaurantEntity> getRestByCity(
			@RequestParam(value = "long")Double longitud,
			@RequestParam(value = "lat")Double latitud,
			@RequestParam(value = "dist")Integer dist
			) {
		List<RestaurantEntity> arrayAll = new ArrayList<>();
		arrayAll = restaurantRepository.findAll();
		return (List<RestaurantEntity>) arrayAll.stream().filter((rt -> calculateDist(rt.getCoordenada_x(),
				rt.getCoordenada_y(), latitud, longitud) < dist)).collect(Collectors.<RestaurantEntity> toList());
	}
		   
	@RequestMapping(value = "/search", method = RequestMethod.GET)
	public @ResponseBody List<RestaurantEntity> getByAttr(
			@RequestParam(value = "nombre",required = false)String nombre,
			@RequestParam(value = "certificado_face", required = false)Boolean certificado_face,
			@RequestParam(value = "tapas", required = false)Boolean tapas,
			@RequestParam(value = "cerveza", required = false)Boolean cerveza,
			@RequestParam(value = "restaurante", required = false)Boolean restaurante,
			@RequestParam(value = "precio", required = false)Integer precio,
			@RequestParam(value = "puntuacion", required = false)Integer puntuacion,
			@RequestParam(value = "porcentaje_gluten_free", required = false)Integer porcentaje_gluten_free
			) {
		List<RestaurantEntity> arrayIdsNombre = new ArrayList<>();
		List<RestaurantEntity> arrayIdsCertificadoFace = new ArrayList<>();
		List<RestaurantEntity> arrayIdsTapas = new ArrayList<>();
		List<RestaurantEntity> arrayIdsCerveza = new ArrayList<>();
		List<RestaurantEntity> arrayIdsRestaurante = new ArrayList<>();
		List<RestaurantEntity> arrayIdsPrecio = new ArrayList<>();
		List<RestaurantEntity> arrayIdsPuntuacion = new ArrayList<>();
		List<RestaurantEntity> arrayIdsPorcentaje_GF = new ArrayList<>();
		List<RestaurantEntity> total = new ArrayList<>();
		Boolean notFirstChange = true;
		
		
		if(nombre != null) {
			
			arrayIdsNombre = restaurantRepository.findByNombre(nombre);			
			notFirstChange = true;
			if(notFirstChange){
				total = arrayIdsNombre;
			} else {
				total = compareArrays(total, arrayIdsNombre);
			}
			notFirstChange = false;
		}
		if(certificado_face != null) {
			arrayIdsCertificadoFace = restaurantRepository.findByCertificadoFace(certificado_face);
			if(notFirstChange){
				total = arrayIdsCertificadoFace;
			} else {
				total = compareArrays(total, arrayIdsCertificadoFace);
			}
			notFirstChange = false;
		}
		if(tapas != null) {
			arrayIdsTapas = restaurantRepository.findByTapas(tapas);
			if(notFirstChange){
				total = arrayIdsTapas;
			} else {
				total = compareArrays(total, arrayIdsTapas);
			}
			notFirstChange = false;
		}
		if(cerveza != null) {
			arrayIdsCerveza = restaurantRepository.findByCerveza(cerveza);
			if(notFirstChange){
				total = arrayIdsCerveza;
			} else {
				total = compareArrays(total, arrayIdsCerveza);
			}
			notFirstChange = false;
		}
		if(restaurante != null) {
			arrayIdsRestaurante = restaurantRepository.findByRestaurante(restaurante);
			if(notFirstChange){
				total = arrayIdsRestaurante;
			} else {
				total = compareArrays(total, arrayIdsRestaurante);
			}
			notFirstChange = false;
		}
		if(precio != null) {
			arrayIdsPrecio = restaurantRepository.findByPrecio(precio);
			if(notFirstChange){
				total = arrayIdsPrecio;
			} else {
				total = compareArrays(total, arrayIdsPrecio);
			}
			notFirstChange = false;
		}
		if(puntuacion != null) {
			arrayIdsPuntuacion = restaurantRepository.findByPuntuacion(puntuacion);
			if(notFirstChange){
				total = arrayIdsPuntuacion;
			} else {
				total = compareArrays(total, arrayIdsPuntuacion);
			}
			notFirstChange = false;
		}
		if(porcentaje_gluten_free != null) {
			arrayIdsPorcentaje_GF = restaurantRepository.findByPorcentaje_GF(porcentaje_gluten_free);
			if(notFirstChange){
				total = arrayIdsPorcentaje_GF;
			} else {
				total = compareArrays(total, arrayIdsPorcentaje_GF);
			}
			notFirstChange = false;
		}
		if(notFirstChange) {
			total = restaurantRepository.findAll();
		}
	return total;
	}
	
	@RequestMapping(value = "/addComment", method = RequestMethod.POST, consumes="application/json")
	public @ResponseBody UserEntity addComment(
			@RequestBody CommentEntity body) {
		commentRepository.addComment(body.getRestaurante().getId_restaurante(),
				body.getUsuario().getId_usuario(),
				body.getDescripcion(),
				body.getPuntuacion());
	return null;
	}
	
	
	@RequestMapping(value = "/deleteComment", method = RequestMethod.DELETE, consumes="application/json")
	public @ResponseBody void deleteComment(
			@RequestBody CommentEntity body) {
		commentRepository.deleteComment(body.getId_comentario());
	}
	
	@RequestMapping(value = "/updateComment", method = RequestMethod.POST, consumes="application/json")
	public @ResponseBody void updateComment(
			@RequestBody CommentEntity body) {
		commentRepository.updateComment(body.getId_comentario(),
				body.getDescripcion(),
				body.getPuntuacion());
	}

	public List<RestaurantEntity> compareArrays(List<RestaurantEntity> array1, List<RestaurantEntity> array2) {
		List<RestaurantEntity> common = new ArrayList<>();
		for (int i = 0; i < array1.size(); i++)
        {
            for (int j = 0; j < array2.size(); j++)
            {
            	Long id1 = array1.get(i).getId_restaurante();
            	Long id2 = array2.get(j).getId_restaurante();
            	
                if(id1 == id2)
                {
                	common.add(array1.get(i));
                 }
            }
        }
		return common;
		
	}
	
	public Double calculateDist (Double lat1, Double lon1, Double lat2, Double lon2){ 
		Double R = 6378.137; // Radius of earth in KM
		Double dLat = (lat2 * Math.PI / 180) - (lat1 * Math.PI / 180);
		Double dLon = (lon2 * Math.PI / 180) - (lon1 * Math.PI / 180);
		Double a = (Math.sin(dLat/2) * Math.sin(dLat/2)) +
		(Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) *
		Math.sin(dLon/2) * Math.sin(dLon/2));
		Double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
		Double d = R * c;
		return d * 1000; // meters
		}
	
}